import unittest
from dyndict import dyndict


class TestDynDict(unittest.TestCase):
    def setUp(self):
        self.dynd = dyndict()

    def tearDown(self):
        del self.dynd

    def test_adding_dictionary(self):
        self.dynd += {'a': 1}
        self.assertEqual(self.dynd, {'a': 1})
        self.dynd += {'b': 2}
        self.assertEqual(self.dynd, {'a': 1, 'b': 2})

    def test_adding_nested_dictionary(self):
        self.dynd += {'a': {}}
        self.assertEqual(self.dynd, {'a': {}})
        self.dynd += {'a': {'b': {}}}
        self.assertEqual(self.dynd, {'a': {'b': {}}})
        self.dynd += {'a': {'b': {'c': 3}}}
        self.assertEqual(self.dynd, {'a': {'b': {'c': 3}}})

    def test_adding_list(self):
        self.dynd += {'a': []}
        self.assertEqual(self.dynd, {'a': []})
        self.dynd += {'a': [1, 2, 3]}
        self.assertEqual(self.dynd, {'a': [1, 2, 3]})
        self.dynd += {'a': [{'b': 2}]}
        self.assertEqual(self.dynd, {'a': [1, 2, 3, {'b': 2}]})
        self.dynd += {'a': [{'b': 2}]}
        self.assertEqual(self.dynd, {'a': [1, 2, 3, {'b': 2}, {'b': 2}]})

    def test_adding_set(self):
        self.dynd += {'a': set()}
        self.assertEqual(self.dynd, {'a': set()})
        self.dynd += {'a': {1, 2, 3}}
        self.assertEqual(self.dynd, {'a': {1, 2, 3}})
        self.dynd += {'a': {1, 2, 3}}
        self.assertEqual(self.dynd, {'a': {1, 2, 3}})
        self.dynd += {'a': {4, 5, 6}}
        self.assertEqual(self.dynd, {'a': {1, 2, 3, 4, 5, 6}})

    def test_overwriting_int(self):
        self.dynd += {'a': -1}
        self.assertEqual(self.dynd, {'a': -1})
        self.dynd += {'a': 1}
        self.assertEqual(self.dynd, {'a': 1})

    def test_overwriting_string(self):
        self.dynd += {'a': 'abc'}
        self.assertEqual(self.dynd, {'a': 'abc'})
        self.dynd += {'a': 'def'}
        self.assertEqual(self.dynd, {'a': 'def'})

    def test_init_with_dict(self):
        self.dynd = dyndict({'a': 1})
        self.assertEqual(self.dynd, {'a': 1})

    def test_refer_by(self):
        self.dynd = dyndict(refer_by='name')
        self.dynd += {'list': [{'name': 'a', 'x': 0}, {'name': 'b', 'x': 1}]}
        self.assertEqual(self.dynd, {'list': [{'name': 'a', 'x': 0}, {'name': 'b', 'x': 1}]})
        self.dynd += {'list': [{'name': 'a', 'y': 2}, {'name': 'b', 'y': 3}]}
        self.assertEqual(self.dynd, {'list': [{'name': 'a', 'x': 0, 'y': 2}, {'name': 'b', 'x': 1, 'y': 3}]})


if __name__ == '__main__':
    unittest.main()



import copy


class dyndict(dict):
    def __init__(self, dictionary:dict=dict(), refer_by=None):
        self.update(dictionary)
        self.refer_by = refer_by

    def __add__(self, b):
        return self._add_primitive(self, b)

    def overwrite(self, dictionary):
        self.dictionary = self._overwrite_primitive(self, dictionary)

    def _add_primitive(self, a, b):
        try:
            if isinstance(a, dict) or isinstance(b, dict):
                raise TypeError
            if isinstance(a, list) or isinstance(b, list):
                raise TypeError
            if isinstance(a, str) and isinstance(b, str):
                return b
            if (isinstance(a, int) or isinstance(a, float)) and (isinstance(b, int) or isinstance(b, float)):
                return b
            return a + b
        except TypeError:
            if isinstance(a, dict) and isinstance(b, dict):
                c = copy.deepcopy(a)
                for i in b:
                    if i in c:
                        c[i] = self._add_primitive(c[i], b[i])
                    else:
                        c[i] = copy.deepcopy(b[i])
                return c
            elif isinstance(a, set) and isinstance(b, set):
                c = copy.deepcopy(a)
                c.update(b)
                return c
            elif isinstance(a, list) and isinstance(b, list):
                c = copy.deepcopy(a)
                d = copy.deepcopy(b)
                e = c + d
                name_dict = dict()
                for i in range(0, len(e)):
                    if isinstance(e[i], dict) and 'name' in e[i]:
                        if e[i]['name'] in name_dict:
                            e[name_dict[e[i]['name']]] = self._add_primitive(e[name_dict[e[i]['name']]], e[i])
                        else:
                            name_dict[e[i]['name']] = i
                name_set = set()
                n = 0
                while n < len(e):
                    if isinstance(e[n], dict) and 'name' in e[n]:
                        if e[n]['name'] in name_set:
                            e.pop(n)
                        else:
                            name_set.add(e[n]['name'])
                            n += 1
                    else:
                        n += 1
                return e
            elif isinstance(a, type(None)) and isinstance(b, type(None)):
                return None
            else:
                print(type(a))
                print(type(b))
                raise NotImplementedError

    def _overwrite_primitive(self, a, b):
        if type(a) != type(b):
            return copy.deepcopy(b)
        elif isinstance(a, dict):
            c = copy.deepcopy(a)
            for i in b:
                if i in c:
                    c[i] = self._overwrite_primitive(c[i], b[i])
                else:
                    c[i] = copy.deepcopy(b[i])
            return c
        else:
            return copy.deepcopy(b)



import setuptools
import subprocess

with open("README.md", "r") as fh:
    long_description = fh.read()

gitcommand = subprocess.run(["git", "describe", "--always", "--abbrev=0"], stdout=subprocess.PIPE, universal_newlines=True)
version = gitcommand.stdout.strip("\n")

setuptools.setup(
    name="dyndict",
    version=version,
    author="Paul (Kyunghan) Lee",
    author_email="contact@paullee.dev",
    description="Dynamic Dictionary Class",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hillcrestpaul0719/dyndict",
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires='>=3.5',
)
